
cmdargs = commandArgs(trailingOnly=T);


MaxNode = as.numeric(cmdargs[1]);
maskFile = cmdargs[2];
Outputdirectory = cmdargs[3];
SNP=cmdargs[4];
Ncov= as.numeric(cmdargs[5]);
covariates= cmdargs[6];

q= as.numeric(cmdargs[7]);

#N=Nx*Ny*Nz

#######################################################################
#Calculate FDR threshold
FDR = function(p,q) {
	p = sort(p);
	V = length(p);
	I = seq(1,V);
	cVID = 1;
	if (length(which(p <= I/V*q/cVID))>0) {
		pID = p[max(which(p <= I/V*q/cVID))];
		} else {
			pID = -Inf;
		}
		return(pID);
	}
#
#######################################################################

Outputdirectory = paste(Outputdirectory,'/',sep='');
#dir.create(Outputdirectory);

Inputdirectory = paste(Outputdirectory,'vRFX/',sep='');
cat("   Input directory:",Inputdirectory,"\n");

library(Rniftilib)
niiMask=nifti.image.read(as.character(maskFile), read_data=1, rm.NaN=TRUE)
mask=as.matrix(niiMask[,,])
Nx=niiMask$dim[1]
Ny=niiMask$dim[2]
Nz=niiMask$dim[3]
indx=which(mask!=0)
L=length(indx);

N=Nx*Ny*Nz

filesN=c("Bvals","Pvals","SEvals")

endNode=MaxNode
done=0
while (done==0) {
	infile<-paste(Inputdirectory,'/',SNP,'_',as.character(filesN[2]),"_node",as.character(endNode),".txt",sep="");
	if (file.exists(infile)){
		endNode=endNode
		done=1
		}else{endNode=endNode-1
		}
	}

	nim=nifti.image.new()
	nim$dim<-c(Nx,Ny,Nz)
	nifti.image.setdatatype(nim,16)

	for (i in 1:length(filesN)) {


		input<-matrix(0,nrow=1,ncol=N)
		for (node in 1:endNode) {

			L=length(indx);
			numV=ceiling(L/MaxNode);
			minV= numV*(node-1)+1

			maxV=min(numV*node,L)
			Lset=maxV-minV +1
			LsetIndx=indx[minV:maxV];

			infile<-paste(Inputdirectory,'/',SNP,'_',as.character(filesN[i]),"_node",as.character(node),".txt",sep="");
	#print(infile)
	input1=scan(infile, quiet=T);
	#print(input1)
	input[,LsetIndx]=input1;
}
p<-NULL
outfile=paste(Outputdirectory,'/',SNP,'_',as.character(filesN[i]),sep="");
p=array(input,dim=c(Nx,Ny,Nz))
nim[,,]=p;
nifti.set.filenames(nim, outfile)
nifti.image.write(nim)
system(paste("gzip ", nim$iname))


if (i==2) {
	zz <- file(paste(Outputdirectory,'/',SNP,"_output_file.txt",sep=""),"w")
	writeLines(paste('   Mask has ',L,'voxels'),con=zz,sep="\n")
	threshold1 = FDR(p[indx],q);
	if (threshold1 != -Inf) {
		cat('   Critical P-value which controls FDR at q=',q,' is',threshold1,'\n');
		cat('  ',length(which(p[indx]<=threshold1)),'voxels survive the FDR threshold\n');
		writeLines(paste('   Mask has ',L,'voxels'),con=zz,sep="\n")
		writeLines(paste('   Critical P-value which controls FDR at q=',q,' is',threshold1),con=zz,sep="\n")
		writeLines(paste('  ',length(which(p[indx]<=threshold1)),'voxels survive the FDR threshold'),con=zz,sep="\n")

		} else {
			cat('   No P-value controls FDR at q=0.05\n');
			writeLines(paste('   No P-value controls FDR at q',q),con=zz,sep="\n")

		}
		cat('  ',length(which(p[indx]<=0.05)),'voxels survive the P<0.05 threshold\n');
		cat('   The minimum P-value is',min(p[indx]),'\n');
		writeLines(paste('  ',length(which(p[indx]<=0.05)),'voxels survive the P<0.05 threshold'),con=zz,sep="\n")
		writeLines(paste('   The minimum P-value is',min(p[indx])),con=zz,sep="\n")
		close(zz)
	}
}

##print out covariate info
#############################
nim=nifti.image.new()
nim$dim<-c(Nx,Ny,Nz)
nifti.image.setdatatype(nim,16)
if (Ncov >0 ){

	parsedCov=parse(text=covariates);

	for (nc in 1:Ncov) {
		zz <- file(paste(Outputdirectory,'/',parsedCov[nc],"_output_file.txt",sep=""),"w")
		for (i in 1:length(filesN)) {
			if (i==1){
				input<-matrix(0,nrow=1,ncol=N)
				} else { input<-matrix(1,nrow=1,ncol=N);
				}
				for (node in 1:endNode) {
					L=length(indx);
					numV=ceiling(L/MaxNode);
					minV= numV*(node-1)+1
					maxV=min(numV*node,L)
					Lset=maxV-minV +1
					LsetIndx=indx[minV:maxV];

					infile<-paste(Inputdirectory,'/',parsedCov[nc],'_',as.character(filesN[i]),"_node",as.character(node),".txt",sep="");

					input1=scan(infile, quiet=T);

					input[,LsetIndx]=input1;
				}
				p<-NULL
				outfile=paste(Outputdirectory,'/',as.character(parsedCov[nc]),'_',as.character(filesN[i]),sep="");
				p=array(input,dim=c(Nx,Ny,Nz))
				nim[,,]=p;
				nifti.set.filenames(nim, outfile)
				nifti.image.write(nim)
				system(paste("gzip ", nim$iname))



			}
			threshold2 = -Inf
			threshold2 = FDR(p[indx],q);
			print(as.character(parsedCov[nc]))
			writeLines(as.character(parsedCov[nc]),con=zz,sep="\n")
			if (threshold2 > 0) {
				cat('   Critical P-value which controls FDR at q=',q,' is',threshold2,'\n');
				cat('  ',length(which(p[indx]<= threshold2)),'voxels survive the FDR threshold\n');
				writeLines(paste('   Critical P-value which controls FDR at  q=',q,' is',threshold2),con=zz,sep="\n")
				writeLines(paste('  ',length(which(p[indx]<= threshold2)),'voxels survive the FDR threshold'),con=zz,sep="\n")
				} else {
					cat('   No P-value controls FDR at  q=',q,' \n');
					writeLines(paste('   No P-value controls FDR at q=',q),con=zz,sep="\n")
				}
				cat('  ',length(which(p[indx]<=0.05)),'voxels survive the P<0.05 threshold\n');
				cat('   The minimum P-value is',min(p),'\n');
				writeLines(paste('  ',length(which(p[indx]<=0.05)),'voxels survive the P<0.05 threshold'),con=zz,sep="\n")
				writeLines(paste('   The minimum P-value is',min(p[indx])),con=zz,sep="\n")
				close(zz)
			}
		}
