cmdargs = commandArgs(trailingOnly=T);

grp=cmdargs[1] # 1 snp 
Outputdirectory=cmdargs[2];
maskFile=cmdargs[3];

node=cmdargs[4];
nodeN=as.numeric(cmdargs[4]);
MaxNode=as.numeric(cmdargs[5]);

#########
library(Rniftilib)
library(nlme)
library(kinship)
source('lmekin.R')
#########

OutputdirectoryEX = paste(Outputdirectory,'/vRFX/',sep='');
DesignMatrix=read.table(paste(Outputdirectory,'/DesignMatrix.txt',sep=''),sep="\t",header=T,blank.lines.skip = TRUE)

pheno_covariatesforRFX=as.matrix(read.table(paste(Outputdirectory,'/pheno_covars.txt',sep=''),sep="\t",header=T,blank.lines.skip = TRUE))
origcolnames=colnames(DesignMatrix)

if ( grp=="none" || grp=="0" || grp =="no") {
    imgpaths=DesignMatrix$imgpaths
    group=NULL

    } else {
	group=DesignMatrix$subjectID
	K=as.matrix(read.table(paste(Outputdirectory,'/KinshipMatrix.txt',sep=''),sep="\t",header=F,blank.lines.skip = TRUE))
	rownames(K)=group ;#DesignMatrix$subjectID;
	colnames(K)=group #DesignMatrix$subjectID;
}

phencovcolnames = colnames(pheno_covariatesforRFX)
Ncov=length(phencovcolnames) - 1;
print(phencovcolnames)
numsubjects = length(DesignMatrix$subjectID);
imgpaths=DesignMatrix$imgpaths

pheno=pheno_covariatesforRFX[,1];
phenotype=as.character(phencovcolnames[1])

### voxel wise specific -- break down of mask to chunks for parallelization
niiMask=nifti.image.read(as.character(maskFile), read_data=1, rm.NaN=TRUE)
mask=as.matrix(niiMask[,,])

Nx=niiMask$dim[1]
Ny=niiMask$dim[2]
Nz=niiMask$dim[3]

indx=which(mask!=0)

L=length(indx);

numV<-ceiling(L/MaxNode);
minV<-numV*(nodeN-1)+1
maxV<-min(numV*nodeN,L)

LsetIndx=indx[minV:maxV];

outslice = matrix(1,nrow=length(LsetIndx),ncol=1);
outsliceB = matrix(0,nrow=length(LsetIndx),ncol=1);
outsliceSE = matrix(0,nrow=length(LsetIndx),ncol=1);
C_outslice = matrix(1,nrow=length(LsetIndx),ncol=Ncov);
C_outsliceB = matrix(0,nrow=length(LsetIndx),ncol=Ncov);
C_outsliceSE = matrix(0,nrow=length(LsetIndx),ncol=Ncov);


# a matrix of info per subject to runfor this node
ROIval<-matrix(0,nrow=numsubjects,ncol=length(LsetIndx))

for (sub in 1: numsubjects) {	

	nii=nifti.image.read(as.character(imgpaths[sub]), read_data=1, rm.NaN=TRUE)
	img=as.matrix(nii[,,])
	
	Nx_i=nii$dim[1]
	Ny_i=nii$dim[2]
	Nz_i=nii$dim[3]
	
	if ( (Nx_i != Nx) || (Ny_i != Ny) || (Nz_i != Nz)) {
		stop('Please make sure your mask and your images are in the same space. Right now the dimensions do not match')	
	}

	img<-as.matrix(img);
	ROIval[sub,]<-img[LsetIndx]
}

for (vox in 1: length(LsetIndx)) {
	print(vox)
	Ys=ROIval[,vox]

	if ( grp=="none" || grp=="0" || grp =="no") {
 
		model=lm(Ys~pheno_covariatesforRFX)
		summarymodel <- summary(model)
		outslice[vox] = summarymodel$coefficients[2,4]
		outsliceB[vox] = summarymodel$coefficients[2,1]
		outsliceSE[vox] = summarymodel$coefficients[2,2]

		if (Ncov > 0 ) {
			for (nc in 1:Ncov){
				C_outslice[vox,nc] = summarymodel$coefficients[2+nc,4]
				C_outsliceB[vox,nc] = summarymodel$coefficients[2+nc,1] 
				C_outsliceSE[vox,nc] = summarymodel$coefficients[2+nc,2]
				}
		}
	} else {
		model=lmekin(Ys~pheno_covariatesforRFX,random=~1|group,varlist=list(as.matrix(K)))
		outslice[vox] = model$ctable[2,4]
		outsliceB[vox] = model$ctable[2,1] 
		outsliceSE[vox] = model$ctable[2,2] 

		if (Ncov > 0 ) {
			for (nc in 1:Ncov){
				C_outslice[vox,nc] = model$ctable[2+nc,4]
				C_outsliceB[vox,nc] = model$ctable[2+nc,1] 
				C_outsliceSE[vox,nc] = model$ctable[2+nc,2]
			}
		}
	}
}

write.table(outslice,paste(OutputdirectoryEX,'/',phenotype,'_Pvals_node',node,'.txt',sep=''),sep="\t",row.names=F,col.names=F);
write.table(outsliceB,paste(OutputdirectoryEX,'/',phenotype,'_Bvals_node',node,'.txt',sep=''),sep="\t",row.names=F,col.names=F);
write.table(outsliceSE,paste(OutputdirectoryEX,'/',phenotype,'_SEvals_node',node,'.txt',sep=''),sep="\t",row.names=F,col.names=F);

if (Ncov > 0 ) {
	for (nc in 1:Ncov){
		write.table(C_outslice[,nc],paste(OutputdirectoryEX,'/',phencovcolnames[nc+1],'_Pvals_node',node,'.txt',sep=''),sep="\t",row.names=F,col.names=F);
		write.table(C_outsliceB[,nc],paste(OutputdirectoryEX,'/',phencovcolnames[nc+1],'_Bvals_node',node,'.txt',sep=''),sep="\t",row.names=F,col.names=F);
		write.table(C_outsliceSE[,nc],paste(OutputdirectoryEX,'/',phencovcolnames[nc+1],'_SEvals_node',node,'.txt',sep=''),sep="\t",row.names=F,col.names=F);
	}
}


close(zz)